@@ -55,7 +55,11 @@ static void
 print_mime_struct (GMimeObject *part, int depth)
 {
 	const GMimeContentType *type;
+	GMimeMultipart *multipart;
+	GMimeMessagePart *mpart;
+	GMimeObject *subpart;
 	gboolean has_md5;
+	int i, n;
 	
 	print_depth (depth);
 	type = g_mime_object_get_content_type (part);
@@ -64,15 +68,16 @@ print_mime_struct (GMimeObject *part, int depth)
 		 has_md5 ? "; md5sum=" : "\n");
 	
 	if (GMIME_IS_MULTIPART (part)) {
-		GList *subpart;
+		multipart = (GMimeMultipart *) part;
 		
-		subpart = GMIME_MULTIPART (part)->subparts;
-		while (subpart) {
-			print_mime_struct (subpart->data, depth + 1);
-			subpart = subpart->next;
+		n = g_mime_multipart_get_number (multipart);
+		for (i = 0; i < n; i++) {
+			subpart = g_mime_multipart_get_part (multipart, i);
+			print_mime_struct (subpart, depth + 1);
+			g_object_unref (subpart);
 		}
 	} else if (GMIME_IS_MESSAGE_PART (part)) {
-		GMimeMessagePart *mpart = (GMimeMessagePart *) part;
+		mpart = (GMimeMessagePart *) part;
 		
 		if (mpart->message)
 			print_mime_struct (mpart->message->mime_part, depth + 1);
